package com.mh.web.controller.mqtt;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.MqttSubscription;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.common.utils.StringUtils;
import com.mh.framework.mqtt.service.IMqttTopicService;
import com.mh.system.service.mqtt.IMqttSubscriptionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description mqtt持久化controller
 * @date 2025-02-14 14:07:45
 */
@RestController
@RequestMapping("/mqtt/subs")
public class MqttSubsController extends BaseController {

    @Autowired
    private IMqttSubscriptionService iMqttSubscriptionService;

    @Autowired
    private IMqttTopicService mqttTopicService;

    /**
     * 获取mqtt订阅记录列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('system:mqtt:list')")
    @GetMapping("/list")
    public TableDataInfo list(MqttSubscription mqttSubscription) {
        startPage();
        List<MqttSubscription> list = iMqttSubscriptionService.selectMqttSubList(mqttSubscription);
        return getDataTable(list);
    }

    /**
     * 根据mqtt订阅记录id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:mqtt:query')")
    @GetMapping(value = "/{subsId}")
    public AjaxResult getInfo(@PathVariable String subsId) {
        return success(iMqttSubscriptionService.selectMqttSubById(subsId));
    }

    /**
     * 新增mqtt订阅记录
     */
    @PreAuthorize("@ss.hasPermi('system:mqtt:add')")
    @Log(title = "mqtt订阅记录管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody MqttSubscription mqttSubscription) {
        mqttSubscription.setCreateBy(getUsername());
        String[] subscribedTopics = mqttTopicService.getSubscribedTopics();
        boolean exists = Arrays.stream(subscribedTopics).anyMatch(val -> val.equals(mqttSubscription.getTopic()));
        if (!exists) {
            int i = iMqttSubscriptionService.insertMqttSub(mqttSubscription);
            mqttTopicService.subscribe(mqttSubscription.getTopic(), mqttSubscription.getQos());
            return toAjax(i);
        }
        return AjaxResult.error("topic已存在");
    }

    /**
     * 修改mqtt订阅记录信息
     */
    @PreAuthorize("@ss.hasPermi('system:mqtt:edit')")
    @Log(title = "mqtt订阅记录管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody MqttSubscription mqttSubscription) {
        mqttSubscription.setUpdateBy(getUsername());
        // 判断是否停用订阅
        mqttSubscription.setUpdateTime(new Date());
        int result = iMqttSubscriptionService.updateMqttSub(mqttSubscription);
        MqttSubscription mqttSubscription1 = iMqttSubscriptionService.selectMqttSubById(mqttSubscription.getId());
        if (mqttSubscription.getStatus().equals("1")) {
            mqttTopicService.unsubscribe(mqttSubscription.getTopic());
        } else if (!Objects.equals(mqttSubscription.getQos(), mqttSubscription1.getQos()) && mqttSubscription.getStatus().equals("0")) {
            mqttTopicService.unsubscribe(mqttSubscription.getTopic());
            mqttTopicService.subscribe(mqttSubscription.getTopic(), mqttSubscription.getQos());
        }
        return toAjax(result);
    }

    /**
     * 删除mqtt订阅记录管理
     */
    @PreAuthorize("@ss.hasPermi('system:mqtt:remove')")
    @Log(title = "mqtt订阅记录管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{subsIdIds}")
    public AjaxResult remove(@PathVariable String[] subsIdIds) {
        // 删除订阅也是直接解绑
        for (String acId : subsIdIds) {
            MqttSubscription mqttSubscription = iMqttSubscriptionService.selectMqttSubById(acId);
            mqttTopicService.unsubscribe(mqttSubscription.getTopic());
        }
        return toAjax(iMqttSubscriptionService.deleteMqttSubByIds(subsIdIds));
    }

}
