package com.mh.framework.dealdata.impl;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.TypeReference;
import com.mh.common.constant.Constants;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DeviceReport;
import com.mh.common.core.redis.RedisCache;
import com.mh.common.model.request.AdvantechDatas;
import com.mh.common.model.request.AdvantechReceiver;
import com.mh.common.model.request.OneTwoThreeTempData;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.EnergyThreadPoolService;
import com.mh.common.utils.StringUtils;
import com.mh.framework.dealdata.DataProcessService;
import com.mh.system.mapper.device.DataProcessMapper;
import com.mh.system.mapper.device.DatabaseMapper;
import com.mh.system.service.device.ICollectionParamsManageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.*;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * 数据解析处理入库操作
 *
 * @Author : Rainbow
 * @date : 2023/5/26
 */
@Service
@Slf4j
public class DataProcessServiceImpl implements DataProcessService {

    @Autowired
    DatabaseMapper databaseMapper;

    @Autowired
    RedisCache redisCache;

    @Autowired
    private ICollectionParamsManageService collectionParamsManageService;

    @Autowired
    private DataProcessMapper dataProcessMapper;

    ThreadPoolExecutor threadPoolService = EnergyThreadPoolService.getInstance();

    @Override
    public void insertTempData(OneTwoThreeTempData oneTwoThreeTempData) {
        insertTempDataToDb(oneTwoThreeTempData, "SENSOR_REGISTER", Constants.TEMP);
    }

    private void insertTempDataToDb(OneTwoThreeTempData data, String registerKey, String cacheKey) {
        log.info("{}数据解析入库：{}", registerKey.equals("SENSOR_REGISTER") ? "温湿度传感器" : "其他设备", data);
        if (registerKey.equals("SENSOR_REGISTER")) {
            databaseMapper.createChillerTable();
        } else {
            databaseMapper.createDataTable();
        }
        ArrayList<CollectionParamsManage> entities = new ArrayList<>();

        List<CollectionParamsManage> registers = redisCache.getCacheList(registerKey, CollectionParamsManage.class);
        if (null == registers || registers.isEmpty()) {
            if (registerKey.equals("SENSOR_REGISTER")) {
                registers = collectionParamsManageService.queryCollectionParamsByMtType(Constants.CHILLERS_TYPE);
            } else {
                registers = collectionParamsManageService.queryCollectionParamsByMtType(Constants.OTHER_TYPE);
            }
            redisCache.setCacheList(registerKey, registers);
        }

        // stream流过滤mtNum中包含data.getId()的集合
        List<CollectionParamsManage> collect = registers.stream().filter(entity -> entity.getMtNum().contains(data.getId())).toList();
        // 温湿度传感器命名规则id+1：温度，id+2：湿度
        if (collect.isEmpty()) {
            return;
        }
        Date curTime = new Date();
        for (CollectionParamsManage entity : collect) {
            BigDecimal temp = new BigDecimal(String.valueOf(data.getParams().getCurrentTemperature().getValue()));
            BigDecimal humidity = new BigDecimal(String.valueOf(data.getParams().getCurrentHumidity().getValue()));
            if (("1"+data.getId()).equals(String.valueOf(entity.getMtNum()))) {
                CollectionParamsManage collectionParamsManage = new CollectionParamsManage();
                collectionParamsManage = entity;
                try {
                    collectionParamsManage.setCurValue(temp);
                } catch (NumberFormatException e) {
                    log.error("数值格式解析异常", e);
                    continue;
                }
                collectionParamsManage.setCurTime(curTime);
                entities.add(collectionParamsManage);
            } else if (("2"+data.getId()).equals(String.valueOf(entity.getMtNum()))) {
                CollectionParamsManage collectionParamsManage = new CollectionParamsManage();
                collectionParamsManage = entity;
                try {
                    collectionParamsManage.setCurValue(humidity);
                } catch (NumberFormatException e) {
                    log.error("数值格式解析异常", e);
                    continue;
                }
                collectionParamsManage.setCurTime(curTime);
                entities.add(collectionParamsManage);
            }
        }

        redisCache.setCacheList(cacheKey, entities);

        threadPoolService.execute(() -> {
            if (!entities.isEmpty()) {
                collectionParamsManageService.updateCollectionParamsManages(entities);
            }
        });
    }

    @Override
    public void insertChillerData(AdvantechReceiver data) {
        insertData(data, "CHILLERS_REGISTER", Constants.CHILLERS);
    }

    @Override
    public void insertDeviceData(AdvantechReceiver data) {
        insertData(data, "DEVICES_REGISTER", Constants.DEVICE);
    }

    private void insertData(AdvantechReceiver data, String registerKey, String cacheKey) {
        log.info("{}数据解析入库：{}", registerKey.equals("CHILLERS_REGISTER") ? "冷水机组" : "计量设备", data);
        if (registerKey.equals("CHILLERS_REGISTER")) {
            databaseMapper.createChillerTable();
        } else {
            databaseMapper.createDataTable();
        }
        ArrayList<CollectionParamsManage> entities = new ArrayList<>();

        List<CollectionParamsManage> registers = redisCache.getCacheList(registerKey, CollectionParamsManage.class);
        if (null == registers || registers.isEmpty()) {
            if (registerKey.equals("CHILLERS_REGISTER")) {
                registers = collectionParamsManageService.queryCollectionParamsByMtType(Constants.CHILLERS_TYPE);
            } else {
                registers = collectionParamsManageService.queryCollectionParamsByMtType(Constants.OTHER_TYPE);
            }
            redisCache.setCacheList(registerKey, registers);
        }

        String timeString = data.getTs();
        OffsetDateTime utcDateTime;
        try {
            utcDateTime = OffsetDateTime.parse(timeString, DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ssZ"));
        } catch (DateTimeParseException e) {
            log.error("时间格式解析异常", e);
            return;
        }
        // 2. 转换为中国时区（UTC+8）
        OffsetDateTime chinaDateTime = utcDateTime.withOffsetSameInstant(
                ZoneOffset.ofHours(8)
        );

        // 3. 格式化为目标字符串
        String formattedTime = chinaDateTime.format(
                DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
        );

        // 假设 data 是一个包含 JSON 数据的对象
        List<AdvantechDatas> list = JSON.parseObject(data.getD().toString(), new TypeReference<List<AdvantechDatas>>() {});
        for (AdvantechDatas advantechDatas : list) {
            String tag = advantechDatas.getTag();
            String value = String.valueOf(advantechDatas.getValue());
            log.info("时间: {},tag标签: {},value值: {}", formattedTime, tag, value);
            try {
                if (StringUtils.isBlank(tag)) {
                    continue;
                }
                for (CollectionParamsManage entity : registers) {
                    if (tag.equals(String.valueOf(entity.getOtherName()))) {
                        CollectionParamsManage collectionParamsManage = new CollectionParamsManage();
                        collectionParamsManage = entity;
                        try {
                            collectionParamsManage.setCurValue(new BigDecimal(value));
                        } catch (NumberFormatException e) {
                            log.error("数值格式解析异常", e);
                            continue;
                        }
                        Date date = DateUtils.stringToDate(formattedTime, "yyyy-MM-dd HH:mm:ss");
                        collectionParamsManage.setCurTime(date);
                        entities.add(collectionParamsManage);
                        break;
                    }
                }
            } catch (Exception e) {
                log.error("解析异常", e);
            }
        }

        redisCache.setCacheList(cacheKey, entities);

        threadPoolService.execute(() -> {
            if (!entities.isEmpty()) {
                collectionParamsManageService.updateCollectionParamsManages(entities);
            }
        });
    }

    @Override
    public void insertOtherData(AdvantechReceiver data) {
        log.info("其他设备数据解析入库：{}", data);
    }

    @Override
    public String queryInitValue(String deviceNum) {
        return dataProcessMapper.queryInitValue(deviceNum);
    }

    @Override
    public DeviceReport queryLastValue(String deviceNum, String type) {
        //先查询今年的表，今年的表没有，再查上一年的表
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        int year = calendar.get(Calendar.YEAR);
        String curTable = "";
        String lastTable = "";
        if ("month".equalsIgnoreCase(type) || "year".equalsIgnoreCase(type)) {
            curTable = "data_" + type;
            lastTable = "data_" + type;
        } else {
            curTable = "data_" + type + year;
            lastTable = "data_" + type + (year - 1);
        }
        DeviceReport entity = null;
        entity = dataProcessMapper.queryLastValue(deviceNum, curTable);
        if (entity == null) {
            try {
                entity = dataProcessMapper.queryLastValue(deviceNum, lastTable);
            } catch (Exception e) {
                log.error("没有当前表:{}", lastTable);
                e.printStackTrace();
            }
        }
        return entity;
    }

    @Override
    public void insertDatabase(List<DeviceReport> dataMinList) {
        log.info("插入data_min数据,数据大小==>{}", dataMinList.size());
        Calendar calendar = Calendar.getInstance();
        //时间格式化0和5结尾的时间
        int batchSize = 10;
        // 拿出第一条数据，看看表中是否存在值，存在则进行update操作
        if (dataMinList.size() > 0) {
            DeviceReport deviceReport = dataMinList.getFirst();
            String yearStr = "data_min" + DateUtils.dateToString(deviceReport.getCurTime(), "yyyy-MM-dd HH:mm:ss").substring(0, 4);
            DeviceReport value = dataProcessMapper.isHaveData(deviceReport,yearStr);
            try {
                if (value!= null && value.getId() > 0) {
                    // 存在则进行更新操作
                    BigDecimal bigDecimal = new BigDecimal(deviceReport.getCurValue());
                    BigDecimal usedValue = bigDecimal.subtract(new BigDecimal(value.getLastValue())).setScale(2, RoundingMode.HALF_UP);
                    BigDecimal calcValue = usedValue.multiply(new BigDecimal(value.getRatio())).setScale(2, RoundingMode.HALF_UP);

                    deviceReport.setId(value.getId());
                    int grade = deviceReport.getGrade();
                    //区分瞬时值
                    if (grade == 1) {
                        deviceReport.setUsedValue(deviceReport.getCurValue());
                        deviceReport.setCalcValue(deviceReport.getCurValue());
                    } else {
                        deviceReport.setUsedValue(String.valueOf(usedValue));
                        deviceReport.setCalcValue(String.valueOf(calcValue));
                    }
                    dataProcessMapper.updateDataMinByTime(deviceReport, yearStr);
                } else {
                    // 不存在则进行插入操作
                    dataProcessMapper.insertDataMin(dataMinList.subList(0, 1), yearStr);
                }
            } catch (Exception e) {
                log.error("插入dataMin失败: ", e);
            }
        }
        // 分页查询并插入数据
        for (int i = 1; i < dataMinList.size(); i += batchSize) {
            List<DeviceReport> batchList = dataMinList.subList(i, Math.min(i + batchSize, dataMinList.size()));
            // 执行插入语句
            calendar.setTime(new Date());
            int year = calendar.get(Calendar.YEAR);
            String curTable = "data_min" + year;
            try {
                dataProcessMapper.insertDataMin(batchList, curTable);
            } catch (Exception e) {
                log.error("batchList:{},curTable:{},date: {}, Exception: ",batchList,curTable,LocalTime.now(),e);
            }
            // 如果需要提交事务可以在这里提交
        }
    }

    /**
     * 聚合计算生成小时表数据
     * 1：从分钟表按照小时为一个区间把数据分成很多区间，grade=0表示未处理
     * 2：每个区间只取最高值和当前时间，并格式化当前时间为小时格式小时，小时后面全部为零
     * 3：得到每个区间的最高值，从第一个开始遍历计算，判断小时表有无数据，按年分表可能需要连表查询
     * 4：如果小时表中有关于这个设备的数据，取最后一条的当前读数作为上次读数，当前抄表时间为上次抄表时间
     * 5：如果小时表无数据，则表示此设备数据第一次入库，取当前设备初始值：device_manage取initial_value
     * 6: 开始循环遍历，第一条数据的上一次读数和上一次抄表时间以小时表的最后一条计算或者初始值计算
     * 7：第二条则以第一条的当前读数作为第二条的上一次读数，直到循环结束，存放到集合中
     * 8：批量将集合插入小时表中
     *
     * @return
     */
    @Override
    public Map<String, Object> queryUntreatedData(String type) {
        //判断是否需要连表
        Calendar now = Calendar.getInstance();
        int year = now.get(Calendar.YEAR);
        int month = now.get(Calendar.MONTH) + 1;
        int day = now.get(Calendar.DATE);
        int hour = now.get(Calendar.HOUR_OF_DAY);
        String curTable = "data_" + type + year;
        List<DeviceReport> curResults = null;
        List<DeviceReport> lastResults = null;
        HashMap<String, Object> resultMap = new HashMap<>();
        if (month == 1 && day == 1 && hour == 0) {
            // 是当前年份的1月1日0点到1点,需要拼接表
            System.out.println(year + "年1月1日0点到1点");
            String lastTable = "data_" + type + (year - 1);
            curResults = dataProcessMapper.queryUntreatedData(curTable);
            lastResults = dataProcessMapper.queryUntreatedData(lastTable);

            // 复制数据
            lastResults.addAll(curResults);
            // BeanUtils.copyProperties(curResults, lastResults);
            HashMap<String, List<Long>> map = new HashMap<>();
            List<Long> curIdList = curResults.stream().map(DeviceReport::getId).collect(Collectors.toList());
            List<Long> lastIdList = lastResults.stream().map(DeviceReport::getId).collect(Collectors.toList());
            map.put("lastIds", lastIdList);
            map.put("curIds", curIdList);

            resultMap.put("data", lastResults);
            resultMap.put("idMap", map);

        } else {
            //不需要拼接表
            System.out.println("不是当前年份的1月1日0点到1点");
            curResults = dataProcessMapper.queryUntreatedData(curTable);
            List<Long> ids = curResults.stream().map(DeviceReport::getId).collect(Collectors.toList());
            HashMap<String, List<Long>> map = new HashMap<>();
            map.put("curIds", ids);

            resultMap.put("data", curResults);
            resultMap.put("idMap", map);

        }
        return resultMap;
    }

    @Override
    public void batchInsertTable(List<DeviceReport> hourList, String type) {

    }

    @Override
    public void batchUpdateGrade(Map<String, List<Long>> idMap, String type) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        int year = calendar.get(Calendar.YEAR);
        Set<Map.Entry<String, List<Long>>> entries = idMap.entrySet();
        for (Map.Entry<String, List<Long>> entry : entries) {
            String key = entry.getKey();
            List<Long> value = entry.getValue();
            if (key.equals("curIds")) {
                String tableName = "data_" + type + year;
                // 分批更新数据
                int batchSize = 1000;
                int totalSize = value.size();
                int batchCount = (int) Math.ceil((double) totalSize / batchSize);
                for (int i = 0; i < batchCount; i++) {
                    int startIndex = i * batchSize;
                    int endIndex = Math.min((i + 1) * batchSize, totalSize);
                    List<Long> batchList = value.subList(startIndex, endIndex);
                    dataProcessMapper.batchUpdateGrade(batchList, tableName, 1);
                }
            } else if (key.equals("lastIds")) {
                String lastTable = "data_" + type + (year - 1);
                // 分批更新数据
                int batchSize = 1000;
                int totalSize = value.size();
                int batchCount = (int) Math.ceil((double) totalSize / batchSize);
                for (int i = 0; i < batchCount; i++) {
                    int startIndex = i * batchSize;
                    int endIndex = Math.min((i + 1) * batchSize, totalSize);
                    List<Long> batchList = value.subList(startIndex, endIndex);
                    dataProcessMapper.batchUpdateGrade(batchList, lastTable, 1);
                }
            }
        }
    }

    @Override
    public void batchInsertOrUpdate(List<DeviceReport> dataList, String type) {
        String tableName;
        for (DeviceReport data : dataList) {
            //不分表  月、年
            tableName = "data_" + type;
            int num = 0;
            switch (type) {
                case "month":
                    num = dataProcessMapper.selectDataByMM(tableName, data.getCurTime(), data.getDeviceNum());
                    break;
                case "year":
                    num = dataProcessMapper.selectDataByYY(tableName, data.getCurTime(), data.getDeviceNum());
                    break;
                case "day":
                    tableName = tableName + DateUtils.dateToString(data.getCurTime(), "yyyy");
                    num = dataProcessMapper.selectDataByDD(tableName, data.getCurTime(), data.getDeviceNum());
                    break;
                case "hour":
                    tableName = tableName + DateUtils.dateToString(data.getCurTime(), "yyyy");
                    num = dataProcessMapper.selectDataByHH(tableName, data.getCurTime(), data.getDeviceNum());
                    if (num == 0) {
                        log.info("插入小时数据：{}", data.toString());
                    }
                    break;
            }
            if (num == 0) {
                //insert
                dataProcessMapper.insertTable(data, tableName);
            } else {
                //update
                switch (type) {
                    case "month":
                        dataProcessMapper.updateTableMM(data, tableName);
                        break;
                    case "year":
                        dataProcessMapper.updateTableYY(data, tableName);
                        break;
                    case "day":
                        dataProcessMapper.updateTableDD(data, tableName);
                        break;
                    case "hour":
                        dataProcessMapper.updateTableHH(data, tableName);
                        break;
                }
            }
        }
    }

    @Override
    public Integer queryRatio(String deviceNum) {
        return 0;
    }

    @Override
    public void insertChillerReport(List<CollectionParamsManage> cacheList) {
        // 先根据表具类型进行分组
        Map<String, List<CollectionParamsManage>> groupMap = cacheList
                .stream()
                .collect(Collectors.groupingBy(CollectionParamsManage::getMtType));
        // 开始进行数据遍历
        for (Map.Entry<String, List<CollectionParamsManage>> entry : groupMap.entrySet()) {
            String mtType = entry.getKey();
            List<CollectionParamsManage> dataList = entry.getValue();
            // 再根据mtNum分组
            Map<String, List<CollectionParamsManage>> groupMap1 = dataList
                    .stream()
                    .collect(Collectors.groupingBy(CollectionParamsManage::getMtNum));
            // 开始进行数据遍历
            for (Map.Entry<String, List<CollectionParamsManage>> entry1 : groupMap1.entrySet()) {
                List<CollectionParamsManage> dataList1 = entry1.getValue();
                // 进行数据处理入库操作等
                try {
                    List<CollectionParamsManage> dealList = new ArrayList<>();
                    Map<Date, Optional<CollectionParamsManage>> dateOptionalMap = dealAndInsertChillers(dataList1, mtType);
                    for (Map.Entry<Date, Optional<CollectionParamsManage>> value : dateOptionalMap.entrySet()) {
                        boolean present = value.getValue().isPresent();
                        if (!present) {
                            continue;
                        }
                        dealList.add(value.getValue().get());
                    }
                    // 根据时间排序
                    dealList.sort(Comparator.comparing(CollectionParamsManage::getCurTime));
                    // 批量插入到chillers_data_min表
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(new Date());
                    int year = calendar.get(Calendar.YEAR);
                    String tableName = "chillers_data_min" + year;
                    for (CollectionParamsManage collectionParamsManage : dealList) {
                        // 判断是否存在，存在的话就不插入
                        DeviceReport deviceReport = new DeviceReport();
                        deviceReport.setDeviceNum(collectionParamsManage.getMtNum());
                        deviceReport.setCurTime(collectionParamsManage.getCurTime());
                        if (null == dataProcessMapper.isHaveData(deviceReport, tableName)) {
                            continue;
                        }
                        dataProcessMapper.batchInsertChiller(List.of(collectionParamsManage), tableName);
                    }
//                    int batchSize = 10;
//                    // 分页查询并插入数据
//                    for (int i = 0; i < cacheList.size(); i += batchSize) {
//                        List<CollectionParamsManage> batchList = cacheList.subList(i, Math.min(i + batchSize, cacheList.size()));
//                        // 执行插入语句
//                        dataProcessMapper.batchInsertChiller(batchList, tableName);
//                    }
                } catch (Exception e) {
                    log.error("处理主机参数异常:{}", e);
                }
            }
        }
    }

    /**
     * 排序
     * @param map
     * @return
     */
    public static Map<Date, Optional<CollectionParamsManage>> sortMapByDate(Map<Date, Optional<CollectionParamsManage>> map) {
        return map.entrySet().stream()
                .sorted(Map.Entry.comparingByKey())
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        Map.Entry::getValue,
                        (oldValue, newValue) -> oldValue,
                        LinkedHashMap::new
                ));
    }

    private Map<Date, Optional<CollectionParamsManage>> dealAndInsertChillers(List<CollectionParamsManage> dataList, String mtType) {
        // 格式化时间点，然后取同样时间点的最大值
        Map<Date, Optional<CollectionParamsManage>> collect = dataList.stream()
                .peek(val -> val.setCurTime(DateUtils.stringToDate(DateUtils.getTimeMin(val.getCurTime(), 1), "yyyy-MM-dd HH:mm:ss")))
                .collect(
                        Collectors.groupingBy(
                                CollectionParamsManage::getCurTime,
                                Collectors.maxBy(Comparator.comparing(CollectionParamsManage::getCurValue)))
                );
        return sortMapByDate(collect);
    }

    @Override
    public void batchUpdateRunTime(List<Map<String, String>> runTimeList) {

    }

    @Override
    public void calculateCopByTime(String time) {

    }
}
