package com.mh.framework.mqtt.config;

import com.mh.common.enums.MqttClientOptions;
import com.mh.common.enums.MqttProtocolEnum;
import com.mh.common.enums.MqttUseEnum;
import lombok.Data;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.util.StringUtils;

import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description mqtt连接配置
 * @date 2024-10-29 14:44:51
 */
@Configuration
@Data
@ConfigurationProperties
public class MqttConfig {

    private static Map<MqttUseEnum, MqttClientOptions> mqttSpring;

    public void setMqttSpring(Map<MqttUseEnum, MqttClientOptions> mqtt) {
        MqttConfig.mqttSpring = mqtt;
    }

    /**
     * 获取mqtt基本配置
     * @return
     */
    static MqttClientOptions getBasicMqttClientOptions() {
        if (!mqttSpring.containsKey(MqttUseEnum.BASIC)) {
            throw new Error("请先配置MQTT的基本连接参数，否则无法启动项目");
        }
        return mqttSpring.get(MqttUseEnum.BASIC);
    }

    /**
     * 拼接获取对应mqtt的连接地址
     * @param options
     * @return
     */
    public static String getMqttAddress(MqttClientOptions options) {
        StringBuilder addr = new StringBuilder();
        addr.append(options.getProtocol().getProtocolAddr())
                .append(options.getHost().trim())
                .append(":")
                .append(options.getPort());
        if ((options.getProtocol() == MqttProtocolEnum.WS || options.getProtocol() == MqttProtocolEnum.WSS)
                && StringUtils.hasText(options.getPath())) {
            addr.append(options.getPath());
        }
        return addr.toString();
    }

    public static String getBasicMqttAddress() {
        return getMqttAddress(getBasicMqttClientOptions());
    }

    /**
     * 获取连接参数，注入到spring中
     * @return
     */
    @Bean
    public MqttConnectOptions mqttConnectionOptions() {

        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();

        MqttClientOptions customizeOptions = getBasicMqttClientOptions();
        String basicMqttAddress = getBasicMqttAddress();
        mqttConnectOptions.setServerURIs(new String[]{basicMqttAddress});
        mqttConnectOptions.setUserName(StringUtils.hasText(customizeOptions.getUsername()) ?
                customizeOptions.getUsername() : "");
        mqttConnectOptions.setPassword(StringUtils.hasText(customizeOptions.getPassword()) ?
                customizeOptions.getPassword().toCharArray() : new char[0]);
        // 直接进行自动连接
        mqttConnectOptions.setAutomaticReconnect(true);
        // 时间间隔时间10s
        mqttConnectOptions.setKeepAliveInterval(10);

        return mqttConnectOptions;
    }

    @Bean
    public MqttPahoClientFactory mqttClientFactory() {
        DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
        factory.setConnectionOptions(mqttConnectionOptions());
        return factory;
    }

}
