package com.mh.framework.mqtt.config;

import com.mh.common.constant.ChannelName;
import com.mh.common.enums.MqttClientOptions;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.annotation.IntegrationComponentScan;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.endpoint.MessageProducerSupport;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.integration.mqtt.support.MqttHeaders;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description 入站配置
 * @date 2024-10-29 16:22:10
 */
@Slf4j
@Configuration
@IntegrationComponentScan
public class MqttInboundConfig {

    @Autowired
    private MqttPahoClientFactory mqttClientFactory;

    @Resource(name = ChannelName.INBOUND)
    private MessageChannel inboundChannel;

    private String clientId;

    /**
     * 入站适配器
     * @return
     */
    @Bean(name = "adapter")
    public MessageProducerSupport mqttInbound() {
        MqttClientOptions options = MqttConfig.getBasicMqttClientOptions();
        // 此处初始化的时候，默认订阅了配置文件中已经写好的topic
        // 如果需要订阅多个，可以自己手动订阅，会写一个addTopic()进行添加订阅
        clientId = options.getClientId() + "_consumer_" + System.currentTimeMillis();
        MqttPahoMessageDrivenChannelAdapter adapter = new MqttPahoMessageDrivenChannelAdapter(
                clientId,
                mqttClientFactory,
                options.getInboundTopic().split(","));
        // System.out.println("每一次都会入站适配器吗？"+clientId);
        DefaultPahoMessageConverter converter = new DefaultPahoMessageConverter();
        // 统一是字节处理
        converter.setPayloadAsBytes(true);
        // 设置消息转换器
        adapter.setConverter(converter);
        // 设置qos（quality of service）
        // 0：最多一次传输（消息会丢失），
        // 1：至少一次传输（消息会重复），
        // 2：只有当消息发送成功时才确认（消息不回丢，但延迟高）。
        adapter.setQos(0);
        // 设置在接收已经订阅的主题信息后，发送给哪个通道，具体的发送方法需要翻上层的抽象类
        adapter.setOutputChannel(inboundChannel);
        return adapter;
    }

    /**
     * 默认声明一个消息处理器，用于处理无效的消息
     * @return
     */
    @Bean
    @ServiceActivator(inputChannel = ChannelName.DEFAULT_BOUND)
    public MessageHandler handler() {
        return message -> {
            log.info("The default channel does not handle messages." +
                    "\nTopic: {}" +
                    "\nPayload: {}",
                    message.getHeaders().get(MqttHeaders.RECEIVED_TOPIC),
                    message.getPayload());
        };
    }

    public String getClientId() {
        return clientId;
    }

}
