package com.mh.framework.mqtt.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mh.common.constant.TopicConst;
import com.mh.common.model.response.CommonTopicResponse;
import com.mh.common.model.response.ServiceReply;
import com.mh.framework.mqtt.service.IMqttGatewayService;
import com.mh.framework.mqtt.service.IMqttMsgSenderService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Objects;
import java.util.UUID;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description mqtt不同类型发送消息
 * @date 2024-10-30 14:30:03
 */
@Slf4j
@Service
public class MqttMsgSenderServiceImpl implements IMqttMsgSenderService {

    @Autowired
    private IMqttGatewayService mqttGatewayService;

    @Autowired
    private ObjectMapper mapper;


    /**
     * 发布，默认qos为0，非持久化
     *
     * @param pushMessage
     * @param topic
     */
    @Override
    public void publish(String topic, String pushMessage) {
        synchronized (this) {//注意：这里一定要同步，否则，在多线程publish的情况下，线程会发生死锁，分析见文章最后补充
            try {
                mqttGatewayService.publish(topic, pushMessage, 0);
                log.info("发送主题：{}，消息：{}", topic, pushMessage);
            } catch (Exception e) {
                log.error("发送主题异常：{}，消息：{}", topic, pushMessage, e);
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public void publish(String topic, CommonTopicResponse response) {
        this.publish(topic, 1, response);
    }

    @Override
    public void publish(String topic, int qos, CommonTopicResponse response) {
        try {
            log.info("发送主题：{}，消息：{}", topic, response.toString());
            mqttGatewayService.publish(topic, mapper.writeValueAsBytes(response), qos);
        } catch (JsonProcessingException e) {
            log.error("发送主题：{}，消息：{}", topic, response.toString(), e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public <T> T publishWithReply(Class<T> clazz, String topic, CommonTopicResponse response) {
        return this.publishWithReply(clazz, topic, response, 2);
    }

    @Override
    public <T> T publishWithReply(Class<T> clazz, String topic, CommonTopicResponse response, int retryTime) {
//        AtomicInteger time = new AtomicInteger(0);
//        // Retry three times
//        while (time.getAndIncrement() <= retryTime) {
//            this.publish(topic, response);
//
//            Chan<CommonTopicReceiver<T>> chan = Chan.getInstance();
//            // If the message is not received in 0.5 seconds then resend it again.
//            CommonTopicReceiver<T> receiver = chan.get(response.getTid());
//
//            // Need to match tid and bid.
//            if (Objects.nonNull(receiver) && receiver.getTid().equals(response.getTid()) &&
//                    receiver.getBid().equals(response.getBid())) {
//                if (clazz.isAssignableFrom(receiver.getData().getClass())) {
//                    return receiver.getData();
//                }
//                throw new TypeMismatchException(receiver.getData(), clazz);
//            }
//            // It must be guaranteed that the tid and bid of each message are different.
//            response.setBid(UUID.randomUUID().toString());
//            response.setTid(UUID.randomUUID().toString());
//        }
//        throw new RuntimeException("没有消息接收到");
        return null;
    }

    @Override
    public <T> ServiceReply<T> publishServicesTopic(TypeReference<T> clazz, String sn, String method, Object data, String bid) {
        String topic = TopicConst.THING_MODEL_PRE + TopicConst.PRODUCT + sn + TopicConst.SERVICES_SUF;
        ServiceReply reply = this.publishWithReply(ServiceReply.class, topic,
                CommonTopicResponse.builder()
                        .tid(UUID.randomUUID().toString())
                        .bid(StringUtils.hasText(bid) ? bid : UUID.randomUUID().toString())
                        .timestamp(System.currentTimeMillis())
                        .method(method)
                        .data(Objects.requireNonNull(data, ""))
                        .build());
        if (Objects.isNull(clazz)) {
            return reply;
        }
        // put together in "output"
        if (Objects.nonNull(reply.getInfo())) {
            reply.setOutput(mapper.convertValue(reply.getInfo(), clazz));
        }
        if (Objects.nonNull(reply.getOutput())) {
            reply.setOutput(mapper.convertValue(reply.getOutput(), clazz));
        }
        return reply;
    }

    @Override
    public ServiceReply publishServicesTopic(String sn, String method, Object data, String bid) {
        return this.publishServicesTopic(null, sn, method, data, bid);
    }

    @Override
    public <T> ServiceReply<T> publishServicesTopic(TypeReference<T> clazz, String sn, String method, Object data) {
        return this.publishServicesTopic(clazz, sn, method, data, null);
    }

    @Override
    public ServiceReply publishServicesTopic(String sn, String method, Object data) {
        return this.publishServicesTopic(null, sn, method, data, null);
    }
}
