package com.mh.framework.rabbitmq;

import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Author : Rainbow
 * @date : 2023/5/26
 */
@Configuration
public class RabbitMqConfig {
    /**交换机*/
    public static final String EXCHANGE_NAME = "exchange_eemcs";
    /**主机队列*/
    public static final String QUEUE_CHILLER = "queue_chiller";
    /**主机routing-key*/
    public static final String ROUTING_KEY_CHILLER = "topic.chillers.eemcs.#";
    /**电表、冷量计等设备队列*/
    public static final String QUEUE_DEVICES = "queue_devices";
    /**电表冷量计routing-key*/
    public static final String ROUTING_KEY_DEVICES = "topic.devices.eemcs.#";
    /**其他设备队列*/
    public static final String QUEUE_OTHER = "queue_other";
    /**DDC routing-key*/
    public static final String ROUTING_KEY_OTHER = "topic.other.eemcs.#";

    /**温湿度设备队列*/
    public static final String QUEUE_TEMP = "queue_temp";
    /**温湿度 routing-key*/
    public static final String ROUTING_KEY_TEMP = "topic.temp.eemcs.#";


    /**durable参数表示交换机是否持久化，值为true表示持久化，值为false表示不持久化。
     * 在RabbitMQ中，持久化交换机会被存储在磁盘上以便在服务器重启后恢复，
     * 而非持久化交换机则只存在于内存中，服务器重启后会丢失*/
    /**声明交换机*/
    @Bean(EXCHANGE_NAME)
    public Exchange exchange(){
        return ExchangeBuilder.topicExchange(EXCHANGE_NAME).durable(true).build();
    }

    /**主机队列*/
    @Bean(QUEUE_CHILLER)
    public Queue chillerQueue(){
        return new Queue(QUEUE_CHILLER);
    }

    /**主机队列绑定交换机*/
    @Bean(ROUTING_KEY_CHILLER)
    public Binding chillerBinding(@Qualifier(QUEUE_CHILLER) Queue queue,
                                  @Qualifier(EXCHANGE_NAME) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(ROUTING_KEY_CHILLER).noargs();
    }

    /**计量设备队列*/
    @Bean(QUEUE_DEVICES)
    public Queue devicesQueue(){
        return new Queue(QUEUE_DEVICES);
    }

    /**计量设备绑定交换机*/
    @Bean(ROUTING_KEY_DEVICES)
    public Binding deviceBinding(@Qualifier(QUEUE_DEVICES) Queue queue,
                                 @Qualifier(EXCHANGE_NAME) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(ROUTING_KEY_DEVICES).noargs();
    }

    /**其他设备队列*/
    @Bean(QUEUE_OTHER)
    public Queue otherQueue(){
        return new Queue(QUEUE_OTHER);
    }

    /**其他队列绑定交换机*/
    @Bean(ROUTING_KEY_OTHER)
    public Binding otherBinding(@Qualifier(QUEUE_OTHER) Queue queue,
                                  @Qualifier(EXCHANGE_NAME) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(ROUTING_KEY_OTHER).noargs();
    }

    /**温湿度传感器设备队列*/
    @Bean(QUEUE_TEMP)
    public Queue tempQueue(){
        return new Queue(QUEUE_TEMP);
    }

    /**温湿度传感器队列绑定交换机*/
    @Bean(ROUTING_KEY_TEMP)
    public Binding tempBinding(@Qualifier(QUEUE_TEMP) Queue queue,
                                @Qualifier(EXCHANGE_NAME) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(ROUTING_KEY_TEMP).noargs();
    }

    /**
     * 默认消费者数量1
     * setConcurrentConsumers(10);
     * 最大消费者数量，默认1
     * maxConcurrentConsumers
     * prefetchCount 每个消费者可以拉取的消息，默认250
     * defaultRequeueRejected 未处理消息是否回归队列，true不重新加入，直接丢弃。false重新加入，默认false
     * acknowledgeMode 确认模式 默认为自动确认
     * @param connectionFactory
     * @return
     */
    @Bean
    @ConditionalOnClass
    public SimpleRabbitListenerContainerFactory rabbitListenerContainerFactory(CachingConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        factory.setConnectionFactory(connectionFactory);
        factory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        factory.setPrefetchCount(5);
        factory.setConcurrentConsumers(5);
        factory.setMaxConcurrentConsumers(10);
        factory.setDefaultRequeueRejected(true);
        return factory;
    }

}
