package com.mh.framework.rabbitmq.consumer;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.mh.common.model.request.AdvantechReceiver;
import com.mh.common.model.request.OneTwoThreeTempData;
import com.mh.framework.dealdata.DataProcessService;
import com.mh.framework.rabbitmq.RabbitMqConfig;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.amqp.core.ExchangeTypes;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.Header;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @Author : ljf
 * @date : 2025-02-07
 */
@Component
@Slf4j
public class ReceiveHandler {

    @Autowired
    DataProcessService dataProcessService;

    /**
     * 监听主机参数
     * queues：指定监听的队列名，可以接收单个队列，也可以接收多个队列的数组或列表。
     * containerFactory：指定使用的监听器容器工厂，通常为SimpleRabbitListenerContainerFactory或其子类的实例，也可以使用自定义的工厂类。
     * concurrency：并发处理消息的数量，即开启几个线程来处理消息，默认为1个线程。 与配置SimpleRabbitListenerContainerFactory的setConcurrentConsumers一致
     * priority：消息的消费者优先级，默认为0，可以在MQ的 x-max-priority 参数打开的情况下使用，来指定消息的优先级。
     * exclusive：是否为独占消费，即限制该队列只能由一个消费者进行消费，设置为true时表示是独占消费，默认为false。
     * bindings：指定队列的绑定关系，即交换机和路由键，可以传入多个Binding对象，也可以使用@QueueBinding注解并传入多个@QueueBinding注解实例。
     * id：指定这个监听器容器的唯一标识，必须显式地设置它以启用有状态的 RabbitListenerContainerFactory 实现，以进行更高效的重启尝试，默认为空。
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = RabbitMqConfig.QUEUE_CHILLER, durable = "true"),
            exchange = @Exchange(
                    value = RabbitMqConfig.EXCHANGE_NAME,
                    ignoreDeclarationExceptions = "true",
                    type = ExchangeTypes.TOPIC
            ),
            key = {RabbitMqConfig.ROUTING_KEY_CHILLER}))
    public void receiveChillerData(@Payload String msg, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long tag) throws IOException, InterruptedException {

        try {
            //TODO 开启多线程处理主机数据,如果不通过线程池开启线程来处理，
            // 设置SimpleRabbitListenerContainerFactory中的setConcurrentConsumers(10)数量也可以实现多线程处理
            log.info("MQ消费者：主机数据采集：{}", msg);
            //TODO 数据解析入库操作 msg转成实体类，入库
            AdvantechReceiver chillerData = JSONObject.parseObject(msg, AdvantechReceiver.class);
            dataProcessService.insertChillerData(chillerData);
            channel.basicAck(tag, false);
        } catch (Exception e) {
            //false不进入队列，丢弃，true则回到队列
            log.error("data:{},chillersException:{}", msg, e);
            Thread.sleep(100);
            channel.basicReject(tag, false);
        }

    }

    /**
     * 处理电表冷量计等数据
     *
     * @param msg
     * @param channel
     * @param tag
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = RabbitMqConfig.QUEUE_DEVICES, durable = "true"),
            exchange = @Exchange(
                    value = RabbitMqConfig.EXCHANGE_NAME,
                    ignoreDeclarationExceptions = "true",
                    type = ExchangeTypes.TOPIC
            ),
            key = {RabbitMqConfig.ROUTING_KEY_DEVICES}
    ))
    public void receiveDeviceData(@Payload String msg, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long tag) throws InterruptedException, IOException {
        try {
            //TODO 处理电表等数据
            log.info("MQ消费者：计量设备数据采集：{}", msg);
            //TODO 数据解析入库操作 msg转成实体类，入库
            AdvantechReceiver deviceData = JSONObject.parseObject(msg, AdvantechReceiver.class);
            dataProcessService.insertDeviceData(deviceData);
            // 正常执行，手动确认ack
            channel.basicAck(tag, false);
        } catch (Exception e) {
            log.error("data:{},deviceException:{}", msg, e);
            Thread.sleep(100);
            channel.basicAck(tag, false);

        }
    }

    /**
     * 处理其他设备数据
     *
     * @param msg
     * @param channel
     * @param tag
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = RabbitMqConfig.QUEUE_OTHER, durable = "true"),
            exchange = @Exchange(
                    value = RabbitMqConfig.EXCHANGE_NAME,
                    ignoreDeclarationExceptions = "true",
                    type = ExchangeTypes.TOPIC
            ),
            key = {RabbitMqConfig.ROUTING_KEY_OTHER}
    ))
    public void receiveOtherData(@Payload String msg, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long tag) throws InterruptedException, IOException {
        try {
            log.info("MQ消费者：其他设备采集：{}", msg);
            //TODO 数据解析入库操作 msg转成实体类，入库
            AdvantechReceiver OtherData = JSONObject.parseObject(msg, AdvantechReceiver.class);
            dataProcessService.insertOtherData(OtherData);
            // 正常执行，手动确认ack
            channel.basicAck(tag, false);
        } catch (Exception e) {
            log.error("data:{},ddcException:{}", msg, e);
            Thread.sleep(100);
            channel.basicAck(tag, false);
        }
    }

    /**
     * 处理温湿度设备数据
     *
     * @param msg
     * @param channel
     * @param tag
     */
    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = RabbitMqConfig.QUEUE_TEMP, durable = "true"),
            exchange = @Exchange(
                    value = RabbitMqConfig.EXCHANGE_NAME,
                    ignoreDeclarationExceptions = "true",
                    type = ExchangeTypes.TOPIC
            ),
            key = {RabbitMqConfig.ROUTING_KEY_TEMP}
    ))
    public void receiveTempData(@Payload String msg, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long tag) throws InterruptedException, IOException {
        try {
            log.info("MQ消费者：温湿度设备采集：{}", msg);
            //TODO 数据解析入库操作 msg转成实体类，入库
            OneTwoThreeTempData oneTwoThreeTempData = JSONObject.parseObject(msg, OneTwoThreeTempData.class);
            dataProcessService.insertTempData(oneTwoThreeTempData);
            // 正常执行，手动确认ack
            channel.basicAck(tag, false);
        } catch (Exception e) {
            log.error("data:{},ddcException:{}", msg, e);
            Thread.sleep(100);
            channel.basicAck(tag, false);
        }
    }

}
