package com.mh.system.service.device.impl;

import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mh.common.core.domain.dto.HouseGroupDTO;
import com.mh.common.core.domain.dto.IndoorTempDTO;
import com.mh.common.core.domain.dto.TempHisRequest;
import com.mh.common.core.domain.dto.TempHumidityDTO;
import com.mh.common.core.domain.entity.ChillersEntity;
import com.mh.common.core.domain.vo.IndoorTempVO;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.device.DataProcessMapper;
import com.mh.system.service.device.IIndoorTempMonitorService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 室内温湿度查询实现类
 * @date 2025-02-21 16:32:23
 */
@Slf4j
@Service
public class IndoorTempMonitorServiceImpl implements IIndoorTempMonitorService {

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Resource
    private DataProcessMapper dataProcessMapper;

    @Override
    public List<HouseGroupDTO> getIndoorTempByFloor(String systemType, String floorId) throws JsonProcessingException {
        // 查询采集类型是温度的参数,采集参数是12
        List<TempHumidityDTO> collectionParamsManages = collectionParamsManageMapper.selectByParamType(systemType, floorId, "12");
        // 遍历重新设置otherName为温度
        for (TempHumidityDTO tempHumidityDTO : collectionParamsManages) {
            tempHumidityDTO.setOtherName("温度");
        }
        // 湿度采集参数：32
        List<TempHumidityDTO> humidityDTOS = collectionParamsManageMapper.selectByParamType(systemType, floorId, "32");
        // 遍历重新设置otherName为湿度
        for (TempHumidityDTO tempHumidityDTO : humidityDTOS) {
            tempHumidityDTO.setOtherName("湿度");
        }
        // 合并collectionParamsManages和humidityDTOS
        collectionParamsManages.addAll(humidityDTOS);
        return convert(collectionParamsManages);
    }

    /**
     * 格式转换成json
     *
     * @param dataList
     * @return
     * @throws JsonProcessingException
     */
    public static List<HouseGroupDTO> convert(List<TempHumidityDTO> dataList) throws JsonProcessingException {
        // 先按orderNum升序排序原始数据
        dataList.sort(Comparator.comparingInt(TempHumidityDTO::getOrderNum));

        // 使用复合键分组（houseId + houseName）
        Map<String, HouseGroupDTO> groupedMap = dataList.stream()
                .collect(Collectors.groupingBy(
                        data -> {
                            log.info("Processing data: houseId={}, houseName={}", data.getHouseId(), data.getHouseName());
                            return data.getHouseId() + "|" + data.getHouseName();
                        },
                        Collectors.collectingAndThen(
                                Collectors.toList(),
                                list -> {
                                    if (list.size() > 1) {
                                        log.warn("Duplicate entries found for key: {}|{}", list.get(0).getHouseId(), list.get(0).getHouseName());
                                    }
                                    HouseGroupDTO group = new HouseGroupDTO(list.getFirst());
                                    for (TempHumidityDTO data : list) {
                                        group.addDevice(data);
                                    }
                                    return group;
                                }
                        )
                ));

        // 转换为有序列表（按orderNum升序）
        return groupedMap.values().stream()
                .sorted(Comparator.comparingInt(HouseGroupDTO::getOrderNum))
                .collect(Collectors.toList());
    }


    @Override
    public List<IndoorTempVO> getIndoorTempHisByFloorAndHouseId(String systemType, String floorId, String houseId) {
        return List.of();
    }

    @Override
    public List<IndoorTempDTO> getIndoorTempHisByHouseId(TempHisRequest tempHisRequest) {
        List<String> houseIds = tempHisRequest.getHouseIds();
        String systemType = tempHisRequest.getSystemType();
        String startTime = tempHisRequest.getStartTime();
        String endTime = tempHisRequest.getEndTime();
        // 1. 参数校验
        if (houseIds.isEmpty()) {
            log.warn("无效房间ID: {}", houseIds);
            return Collections.emptyList();
        }

        // 2. 准备基础数据
        String tableName = "chillers_data_min" + LocalDateTime.now().getYear();

        // 3. 获取温度数据映射
        Map<String, HouseData> tempData = getHouseDataMapping(houseIds, systemType, "8", "12");
        Map<String, HouseData> humidityData = getHouseDataMapping(houseIds, systemType, "8", "32");

        // 4. 构建结果
        return houseIds.stream()
                .map(houseId -> buildIndoorTempDTO(
                        houseId,
                        tableName,
                        tempData.getOrDefault(houseId, HouseData.EMPTY),
                        humidityData.getOrDefault(houseId, HouseData.EMPTY),
                        startTime,
                        endTime
                ))
                .collect(Collectors.toList());
    }

    // 辅助类封装房屋数据
    private static class HouseData {
        static final HouseData EMPTY = new HouseData("未知房间", "0", Collections.emptyList());

        final String houseName;
        final String orderNum;
        final List<String> cpmIds;

        HouseData(String houseName, String orderNum, List<String> cpmIds) {
            this.houseName = houseName;
            this.orderNum = orderNum;
            this.cpmIds = cpmIds;
        }
    }

    // 获取房屋数据映射
    private Map<String, HouseData> getHouseDataMapping(List<String> houseIds, String systemType, String mtType, String paramType) {
        List<Map<String, String>> rawData = collectionParamsManageMapper.selectByHouseIdAndMtType(houseIds, systemType, mtType, paramType);

        return rawData.stream()
                .filter(map -> map.containsKey("houseId") && map.containsKey("houseName") && map.containsKey("orderNum"))
                .collect(Collectors.toMap(
                        // 确保 houseId 转换为 String
                        map -> String.valueOf(map.get("houseId")),
                        map -> new HouseData(
                                // 确保 houseName 转换为 String
                                String.valueOf(map.get("houseName")),
                                // 确保 orderNum 转换为 String
                                String.valueOf(map.get("orderNum")),
                                rawData.stream()
                                        .filter(m -> String.valueOf(m.get("houseId")).equals(String.valueOf(map.get("houseId"))))
                                        .map(m -> String.valueOf(m.get("cpmId")))
                                        .collect(Collectors.toList())
                        ),
                        (oldVal, newVal) -> oldVal
                ));
    }

    // 构建单个房屋的DTO
    private IndoorTempDTO buildIndoorTempDTO(String houseId, String tableName, HouseData tempData, HouseData humidityData, String startTime, String endTime) {
        List<IndoorTempVO> tempRecords = getProcessedData(tableName, tempData.cpmIds, startTime, endTime);
        List<IndoorTempVO> humidityRecords = getProcessedData(tableName, humidityData.cpmIds, startTime, endTime);

        List<IndoorTempVO> mergedData = mergeTempAndHumidity(tempRecords, humidityRecords);

        return new IndoorTempDTO(
                Integer.parseInt(tempData.orderNum),
                houseId,
                tempData.houseName,
                mergedData
        );
    }

    // 获取处理后的数据
    private List<IndoorTempVO> getProcessedData(String tableName, List<String> cpmIds, String startTime, String endTime) {
        if (cpmIds.isEmpty()) {
            return Collections.emptyList();
        }

        List<IndoorTempVO> rawData = dataProcessMapper.selectHisDataByCodeAndRegisterAddr(tableName, cpmIds, startTime, endTime);

        return rawData.stream()
                .collect(Collectors.groupingBy(
                        IndoorTempVO::getCurTime,
                        () -> new TreeMap<>(this::compareTimeString),
                        Collectors.collectingAndThen(
                                Collectors.toList(),
                                this::calculateAverage
                        )
                ))
                .entrySet().stream()
                .map(entry -> buildResultVO(entry.getKey(), entry.getValue()))
                .collect(Collectors.toList());
    }

    // 合并温度和湿度数据
    private List<IndoorTempVO> mergeTempAndHumidity(List<IndoorTempVO> tempData, List<IndoorTempVO> humidityData) {
        return humidityData.stream()
                .filter(humidity -> tempData.stream().anyMatch(temp -> temp.getCurTime().equals(humidity.getCurTime())))
                .map(humidity -> {
                    IndoorTempVO temp = tempData.stream()
                            .filter(t -> t.getCurTime().equals(humidity.getCurTime()))
                            .findFirst()
                            .orElseThrow();

                    return new IndoorTempVO(
                            humidity.getCurTime(),
                            temp.getIndoorTemp().setScale(1, RoundingMode.HALF_UP),
                            humidity.getIndoorTemp().setScale(1, RoundingMode.HALF_UP)
                    );
                })
                .collect(Collectors.toList());
    }


    // 辅助方法：时间格式校验
    private boolean isValidTimeFormat(String time) {
        return time != null && time.matches("\\d{4}-\\d{2}-\\d{2} \\d{2}");
    }

    // 辅助方法：时间字符串比较
    private int compareTimeString(String t1, String t2) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:00:00");
        LocalDateTime dt1 = LocalDateTime.parse(t1, formatter);
        LocalDateTime dt2 = LocalDateTime.parse(t2, formatter);
        return dt1.compareTo(dt2);
    }

    // 辅助方法：计算平均值
    private BigDecimal calculateAverage(List<IndoorTempVO> list) {
        BigDecimal sum = list.stream()
                .map(IndoorTempVO::getIndoorTemp)
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        return sum.divide(
                new BigDecimal(list.size()),
                2,
                RoundingMode.HALF_UP
        );
    }

    // 辅助方法：构建结果对象
    private IndoorTempVO buildResultVO(String time, BigDecimal avg) {
        IndoorTempVO vo = new IndoorTempVO();
        vo.setCurTime(time);
        vo.setIndoorTemp(avg);
        return vo;
    }

}
