package com.mh.system.service.energy.impl;

import com.alibaba.fastjson2.JSONObject;
import com.mh.common.constant.Constants;
import com.mh.common.core.domain.ColumnData;
import com.mh.common.core.domain.ColumnFilter;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DeviceTypeEnergy;
import com.mh.common.core.domain.vo.EnergyQueryVO;
import com.mh.common.utils.DateUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.energy.EnergyAnalyzeMapper;
import com.mh.system.service.energy.EnergyAnalyzeService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 能源分析实现类
 * @date 2023-12-14 11:50:47
 */
@Service
public class EnergyAnalyzeServiceImpl implements EnergyAnalyzeService {

    @Resource
    private EnergyAnalyzeMapper energyAnalyzeMapper;

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Override
    public JSONObject sysAnalyze(EnergyQueryVO vo) {
        DateUtils.sysEnergyDateChange(vo);
        // 获取参数
        AtomicReference<String> lastTableName = new AtomicReference<>("data_" + vo.getTimeType());
        AtomicReference<String> curTableName = new AtomicReference<>("data_" + vo.getTimeType());
        String timeType = vo.getTimeType();
        List<DeviceTypeEnergy> deviceTypeEnergyEntities = null;
        // 表格数据
        if ("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType)) {
            // 单表
            deviceTypeEnergyEntities = energyAnalyzeMapper.queryOneTable(vo.getStartTime(),
                    vo.getEndTime(),
                    lastTableName.get(),
                    curTableName.get(),
                    DateUtils.getTimeLen(vo.getTimeType()),
                    Constants.METER_TYPE,
                    vo.getSystemType());
        } else {
            lastTableName.set(lastTableName + vo.getStartTime().substring(0, 4));
            curTableName.set(curTableName + vo.getEndTime().substring(0, 4));
            if (lastTableName.get().equalsIgnoreCase(curTableName.get())) {
                // 单表
                deviceTypeEnergyEntities = energyAnalyzeMapper.queryOneTable(vo.getStartTime(),
                        vo.getEndTime(),
                        lastTableName.get(),
                        curTableName.get(),
                        DateUtils.getTimeLen(vo.getTimeType()),
                        Constants.METER_TYPE,
                        vo.getSystemType());
            } else {
                // 多表
                deviceTypeEnergyEntities = energyAnalyzeMapper.queryManyTable(vo.getStartTime(),
                        vo.getEndTime(),
                        lastTableName.get(),
                        curTableName.get(),
                        DateUtils.getTimeLen(vo.getTimeType()),
                        Constants.METER_TYPE,
                        vo.getSystemType());
            }
        }
        if (null == deviceTypeEnergyEntities || deviceTypeEnergyEntities.size() == 0) {
            return new JSONObject();
        }
        // 开始进行分组
        // 通过device_manage的grade进行分组，分为四类：0-9：冷水机，10-19：冷冻泵，20-29：冷却泵，30-39：冷却塔
        HashMap<String, BigDecimal> result = deviceTypeEnergyEntities.stream()
                // 过滤叼总表的数据（deviceType=15）
                .filter(deviceTypeEnergyEntity -> !deviceTypeEnergyEntity.getDeviceType().equals("15"))
                .collect(Collectors.groupingBy(DeviceTypeEnergy::getDeviceTypeRange))
                .entrySet().stream().collect(Collectors.toMap(
                        Map.Entry<String, List<DeviceTypeEnergy>>::getKey,
                        entry -> {
                            List<DeviceTypeEnergy> value = entry.getValue();
                            BigDecimal sum = new BigDecimal("0");
                            for (DeviceTypeEnergy deviceTypeEnergyEntity : value) {
                                sum = sum.add(new BigDecimal(deviceTypeEnergyEntity.getCalcValue()));
                            }
                            return sum;
                        },
                        BigDecimal::add,
                        HashMap::new
                ));
        List<ColumnFilter> results = new ArrayList<>();
        for (String value : Constants.DEVICE_TYPE) {
            ColumnFilter columnFilter = new ColumnFilter();
            columnFilter.setName(value);
            if (result.containsKey(value)) {
                columnFilter.setValue(String.valueOf(result.get(value).setScale(2, RoundingMode.HALF_UP)));
                results.add(columnFilter);
            } else {
                columnFilter.setValue("0.00");
                results.add(columnFilter);
            }
        }

        // 查询各个冷水机组的 0-9
        List<DeviceTypeEnergy> chiller = createDeviceTypeData(deviceTypeEnergyEntities, 0, 0);

        // 查询各个冷冻泵的 10-19
        List<DeviceTypeEnergy> chillerPump = createDeviceTypeData(deviceTypeEnergyEntities, 2, 2);

        // 查询各个冷却泵的 20-29
        List<DeviceTypeEnergy> coolPump = createDeviceTypeData(deviceTypeEnergyEntities, 1, 1);

        // 查询各个冷却塔的 30-39
        List<DeviceTypeEnergy> coolTower = createDeviceTypeData(deviceTypeEnergyEntities, 3, 3);

        // 从device_manage查询不是总表的
        List<CollectionParamsManage> deviceByOther = collectionParamsManageMapper.getDeviceByOther(null);


        if (deviceByOther != null && deviceByOther.size() > 0) {
            // 通过stream进行分组，然后进行重新赋值，封装新对象，重新返回
            for (CollectionParamsManage deviceManageEntity : deviceByOther) {

                // 重新赋值冷水机
                reassignData(chiller, deviceManageEntity);

                // 重新赋值冷冻泵
                reassignData(chillerPump, deviceManageEntity);

                // 重新赋值冷却泵
                reassignData(coolPump, deviceManageEntity);

                // 重新赋值冷却塔
                reassignData(coolTower, deviceManageEntity);
            }
        }
        List<ColumnFilter> chillerData = getResultData(chiller);

        List<ColumnFilter> chillerPumpData = getResultData(chillerPump);

        List<ColumnFilter> coolPumpData = getResultData(coolPump);

        List<ColumnFilter> coolTowerData = getResultData(coolTower);

        JSONObject resultJson = new JSONObject();
        resultJson.put("all", results);
        resultJson.put("chiller", chillerData);
        resultJson.put("chillerPump", chillerPumpData);
        resultJson.put("coolPump", coolPumpData);
        resultJson.put("coolTower", coolTowerData);

        return resultJson;
    }

    @Override
    public List<Map.Entry<String, Object>> deviceAnalyze(EnergyQueryVO vo) {
        DateUtils.sysEnergyDateChange(vo);
        // 获取参数
        List<ColumnFilter> paramsMap = vo.getSearchParams();
        if (paramsMap.isEmpty()) {
            return List.of();
        }
        List<Object> params = paramsMap.stream().map(ColumnFilter::getValue).collect(Collectors.toList());
        AtomicReference<String> lastTableName = new AtomicReference<>("data_" + vo.getTimeType());
        AtomicReference<String> curTableName = new AtomicReference<>("data_" + vo.getTimeType());
        String timeType = vo.getTimeType();
        List<DeviceTypeEnergy> deviceTypeEnergyEntities = null;
        // 表格数据
        if ("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType)) {
            // 单表
            deviceTypeEnergyEntities = energyAnalyzeMapper.queryDeviceOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), params, vo.getSystemType());
        } else {
            lastTableName.set(lastTableName + vo.getStartTime().substring(0, 4));
            curTableName.set(curTableName + vo.getEndTime().substring(0, 4));
            if (lastTableName.get().equalsIgnoreCase(curTableName.get())) {
                // 单表
                deviceTypeEnergyEntities = energyAnalyzeMapper.queryDeviceOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), params, vo.getSystemType());
            } else {
                // 多表
                deviceTypeEnergyEntities = energyAnalyzeMapper.queryDeviceManyTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), params, vo.getSystemType());
            }
        }
        if (null == deviceTypeEnergyEntities || deviceTypeEnergyEntities.isEmpty()) {
            return List.of();
        }
        // 分组拿出各个设备
        Map<String, List<DeviceTypeEnergy>> deviceNumMap = deviceTypeEnergyEntities.stream()
                .sorted(Comparator.comparing(DeviceTypeEnergy::getTimeStr))
                .collect(Collectors.groupingBy(DeviceTypeEnergy::getDeviceNum));

        List<String> title = new ArrayList<>();
        List<String> timeStr = new ArrayList<>();
        List<ColumnData> columnData = new ArrayList<>();
        // 组装对应需要查询的数据
        int i = 0;
        for (Map.Entry<String, List<DeviceTypeEnergy>> stringListEntry : deviceNumMap.entrySet()) {
            title.add(stringListEntry.getKey());
            List<DeviceTypeEnergy> value = stringListEntry.getValue();
            ColumnData column = new ColumnData();
            column.setName(stringListEntry.getKey());
            for (DeviceTypeEnergy deviceTypeEnergy : value) {
                if (i == 0) {
                    timeStr.add(deviceTypeEnergy.getTimeStr());
                } else {
                    break;
                }
            }
            int pageNum = Integer.parseInt(String.valueOf(vo.getPageNum()));
            int pageSize = Integer.parseInt(String.valueOf(vo.getPageSize()));
            if (pageNum != 0) {
                int startIndex = (pageNum-1)*pageSize;
                int endIndex = Math.min(pageNum * pageSize, timeStr.size());
                value = value.subList(startIndex, endIndex);
                column.setValue(getArr(value, timeStr.subList(startIndex, endIndex).toArray(new String[0])));
            } else {
                column.setValue(getArr(value, timeStr.toArray(new String[0])));
            }
            columnData.add(column);
            i++;
        }
        int pageNum = Integer.parseInt(String.valueOf(vo.getPageNum()));
        int pageSize = Integer.parseInt(String.valueOf(vo.getPageSize()));
        Map<String, Object> map = new JSONObject();
        map.put("title", title);
        if (pageNum == 0) {
            map.put("timeStr", timeStr);
            map.put("dataList", columnData);
        } else {
            int startIndex = (pageNum-1) * pageSize;
            int endIndex = Math.min(pageNum * pageSize, timeStr.size());
            if (startIndex > endIndex) {
                return List.of();
            }
            map.put("timeStr", timeStr.subList(startIndex, endIndex));
            map.put("dataList", columnData);
        }
        return new ArrayList<>(map.entrySet());
    }

    private static String[] getArr(List<DeviceTypeEnergy> copLineData, String[] lineTimeStrArr) {
        String[] lineCopArr = new String[lineTimeStrArr.length];
        for (int i = 0; i < lineTimeStrArr.length; i++) {
            int j = i;
            Optional<DeviceTypeEnergy> first = copLineData.stream().filter(s -> lineTimeStrArr[j].equalsIgnoreCase(s.getTimeStr())).findFirst();
            if (first.isPresent()) {
                lineCopArr[i] = first.get().getCalcValue();
            } else {
                lineCopArr[i] = "0.00";
            }
        }
        return lineCopArr;
    }

    private static List<ColumnFilter> getResultData(List<DeviceTypeEnergy> coolTower) {
        return coolTower.stream()
                .sorted(Comparator.comparing(DeviceTypeEnergy::getGrade))
                .map(val -> {
                    ColumnFilter data = new ColumnFilter();
                    data.setName(val.getRemark());
                    BigDecimal calcValue = new BigDecimal(val.getCalcValue()).setScale(2, RoundingMode.HALF_UP);
                    data.setValue(String.valueOf(calcValue));
                    return data;
                }).collect(Collectors.toList());
    }

    private static void reassignData(List<DeviceTypeEnergy> chiller, CollectionParamsManage deviceManageEntity) {
        chiller.stream()
                .filter(val -> val.getDeviceNum().equals(deviceManageEntity.getMtNum()))
                .peek(val -> {
                    val.setDeviceType(deviceManageEntity.getMtType());
                    val.setRemark(deviceManageEntity.getOtherName());
                    val.setGrade(String.valueOf(deviceManageEntity.getGrade()));
                })
                .sorted(Comparator.comparing(DeviceTypeEnergy::getGrade))
                .collect(Collectors.toList());
    }

    private static List<DeviceTypeEnergy> createDeviceTypeData(List<DeviceTypeEnergy> deviceTypeEnergyEntities, int x, int x1) {
        return deviceTypeEnergyEntities.stream()
                .filter(val -> Integer.parseInt(val.getDeviceType()) == x)
                .collect(Collectors.groupingBy(DeviceTypeEnergy::getDeviceNum, Collectors.summarizingDouble(DeviceTypeEnergy::getDoubleCalcValue)))
                .entrySet().stream()
                .map(entry -> {
                    DeviceTypeEnergy deviceTypeEnergy = new DeviceTypeEnergy();
                    deviceTypeEnergy.setCalcValue(String.valueOf(entry.getValue().getSum()));
                    deviceTypeEnergy.setDeviceNum(entry.getKey());
                    return deviceTypeEnergy;
                })
                .collect(Collectors.toList());
    }

}
