package com.mh.system.service.space.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.SpaceTreeSelect;
import com.mh.common.core.domain.TreeSelect;
import com.mh.common.core.domain.entity.*;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.SysParamsMapper;
import com.mh.system.mapper.space.AreaInfoMapper;
import com.mh.system.mapper.space.BuildingInfoMapper;
import com.mh.system.mapper.space.FloorInfoMapper;
import com.mh.system.mapper.space.HouseInfoMapper;
import com.mh.system.service.space.IHouseInfoService;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 房间管理
 * @date 2025-02-19 18:34:42
 */
@Service
public class HouseInfoServiceImpl implements IHouseInfoService {

    @Resource
    private AreaInfoMapper areaInfoMapper;

    @Resource
    private BuildingInfoMapper buildingInfoMapper;

    @Resource
    private FloorInfoMapper floorInfoMapper;

    @Resource
    private HouseInfoMapper houseInfoMapper;

    @Resource
    private SysParamsMapper sysParamsMapper;

    @Override
    public List<HouseInfo> selectHouseInfoList(HouseInfo houseInfo) {
        if (houseInfo == null) {
            return List.of();
        }
        QueryWrapper<HouseInfo> queryWrapper = new QueryWrapper<>();
        // 区域id
        if (!StringUtils.isEmpty(houseInfo.getAreaId())) {
            queryWrapper.eq("area_id", houseInfo.getAreaId());
        }
        // 楼栋id
        if (!StringUtils.isEmpty(houseInfo.getBuildingId())) {
            queryWrapper.eq("building_id", houseInfo.getBuildingId());
        }
        // 楼层id
        if (!StringUtils.isEmpty(houseInfo.getFloorId())) {
            queryWrapper.eq("floor_id", houseInfo.getFloorId());
        }
        // 房间名称
        if (!StringUtils.isEmpty(houseInfo.getHouseName())) {
            queryWrapper.like("house_name", houseInfo.getHouseName());
        }
        // 备注
        if (!StringUtils.isEmpty(houseInfo.getRemark())) {
            queryWrapper.like("remark", houseInfo.getRemark());
        }
        queryWrapper.orderByAsc("order_num");
        return houseInfoMapper.selectList(queryWrapper);
    }

    @Override
    public HouseInfo selectHouseInfoById(String buildingId) {
        return houseInfoMapper.selectById(buildingId);
    }

    @Override
    public int insertHouseInfo(HouseInfo houseInfo) {
        getAreaIdAndBuildingId(houseInfo);
        return houseInfoMapper.insert(houseInfo);
    }

    // 递归查找楼栋节点方法
    private SpaceTreeSelect findNode(List<SpaceTreeSelect> nodes, String targetId, int nodeType) {
        for (SpaceTreeSelect node : nodes) {
            if (node.getNodeType() == nodeType && targetId.equals(node.getId())) {
                return node;
            }
            if (node.getChildren() != null && !node.getChildren().isEmpty()) {
                SpaceTreeSelect found = findNode(node.getChildren(), targetId, nodeType);
                if (found != null) {
                    return found;
                }
            }
        }
        return null;
    }

    @Override
    public int updateHouseInfo(HouseInfo houseInfo) {
        getAreaIdAndBuildingId(houseInfo);
        return houseInfoMapper.updateById(houseInfo);
    }

    private void getAreaIdAndBuildingId(HouseInfo houseInfo) {
        if(StringUtils.isEmpty(houseInfo.getBuildingId())) {
            // 获取完整的空间树结构
            List<SpaceTreeSelect> spaceTree = buildTree(null);

            // 递归查找目标楼栋节点
            SpaceTreeSelect buildNode = findNode(spaceTree, houseInfo.getFloorId(), 3);

            if(buildNode != null && buildNode.getParentId() != null) {
                houseInfo.setBuildingId(buildNode.getParentId());
            } else {
                throw new RuntimeException("未找到对应的楼栋id");
            }
        }
        if(StringUtils.isEmpty(houseInfo.getAreaId())) {
            // 获取完整的空间树结构
            List<SpaceTreeSelect> spaceTree = buildTree(null);

            // 递归查找目标楼栋节点
            SpaceTreeSelect areaNode = findNode(spaceTree, houseInfo.getBuildingId(), 2);

            if(areaNode != null && areaNode.getParentId() != null) {
                houseInfo.setAreaId(areaNode.getParentId());
            } else {
                throw new RuntimeException("未找到对应的区域id");
            }
        }
        if (StringUtils.isEmpty(houseInfo.getFloorId())) {
            throw new RuntimeException("没有对应的楼层id");
        }
    }

    @Override
    public int deleteHouseInfoByIds(String[] buildingIds) {
        if (buildingIds != null && buildingIds.length > 0) {
            for (String buildingId : buildingIds) {
                houseInfoMapper.deleteById(buildingId);
            }
            return buildingIds.length;
        }
        return 0;
    }

    @Override
    public List<SpaceTreeSelect> buildTree(String systemType) {
        QueryWrapper<AreaInfo> areaInfoQueryWrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(systemType)) {
            areaInfoQueryWrapper.eq("system_type", systemType);
        }
        areaInfoQueryWrapper.orderByAsc("order_num");
        List<AreaInfo> areaInfos = areaInfoMapper.selectList(areaInfoQueryWrapper);
        if (areaInfos != null && !areaInfos.isEmpty()) {
            List<BuildingInfo> buildingInfos = buildingInfoMapper.selectList(new QueryWrapper<BuildingInfo>().orderByAsc("order_num"));
            List<FloorInfo> floorInfos = floorInfoMapper.selectList(new QueryWrapper<FloorInfo>().orderByAsc("order_num"));
            List<HouseInfo> houseInfos = houseInfoMapper.selectList(new QueryWrapper<HouseInfo>().orderByAsc("order_num"));
            return buildTree(areaInfos, buildingInfos, floorInfos, houseInfos);
        }
        return List.of();
    }

    public List<SpaceTreeSelect> buildTree(List<AreaInfo> areaInfos,
                                      List<BuildingInfo> buildingInfos,
                                      List<FloorInfo> floorInfos,
                                      List<HouseInfo> houseInfos) {
        // 房间 -> 楼层映射
        Map<String, List<SpaceTreeSelect>> floorMap = houseInfos.stream()
                .map(r -> new SpaceTreeSelect(r.getId(), r.getHouseName(), false, r.getFloorId(), 4))
                .collect(Collectors.groupingBy(SpaceTreeSelect::getParentId));

        // 楼层 -> 楼栋映射
        Map<String, List<SpaceTreeSelect>> buildingMap = floorInfos.stream()
                .map(f -> {
                    SpaceTreeSelect node = new SpaceTreeSelect(f.getId(), f.getFloorName(), false, f.getBuildingId(), 3);
                    node.setChildren(floorMap.getOrDefault(f.getId(), Collections.emptyList()));
                    return node;
                })
                .collect(Collectors.groupingBy(SpaceTreeSelect::getParentId));

        // 楼栋 -> 区域映射
        Map<String, List<SpaceTreeSelect>> areaMap = buildingInfos.stream()
                .map(b -> {
                    SpaceTreeSelect node = new SpaceTreeSelect(b.getId(), b.getBuildingName(), false, b.getAreaId(), 2);
                    node.setChildren(buildingMap.getOrDefault(b.getId(), Collections.emptyList()));
                    return node;
                })
                .collect(Collectors.groupingBy(SpaceTreeSelect::getParentId));

        // 区域 -> 根节点映射
        Map<String, List<SpaceTreeSelect>> map = areaInfos.stream()
                .map(a -> {
                    SpaceTreeSelect node = new SpaceTreeSelect(a.getId(), a.getAreaName(), false, "0", 1);
                    node.setChildren(areaMap.getOrDefault(a.getId(), Collections.emptyList()));
                    return node;
                })
                .collect(Collectors.groupingBy(SpaceTreeSelect::getParentId));
        List<SysParams> sysParams = sysParamsMapper.selectSysParamsList();
        return sysParams.stream().map(r -> {
            SpaceTreeSelect node = new SpaceTreeSelect(r.getId(), r.getProName(), false, null, 0);
            node.setChildren(map.getOrDefault(r.getId(), Collections.emptyList()));
            return node;
        }).collect(Collectors.toList());
    }

}
